<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2007-2010 Rene Nitzsche (rene@system25.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(t3lib_extMgm::extPath('rn_base') . 'class.tx_rnbase.php');
tx_rnbase::load('tx_rnbase_util_DB');
require_once(t3lib_extMgm::extPath('cfc_league_fe') . 'models/class.tx_cfcleaguefe_models_competition_penalty.php');

/**
 * Die Klasse ist in der Lage, Tabellen einer Liga zu berechnen.
 */
class tx_cfcleaguefe_util_LeagueTable  {
  var $_teamData;
  // Daten für die Punktezählung
  var $cfgPointSystem, $cfgPointsDraw, $cfgPointsWin, $cfgPointsLose;
  var $cfgTableType, $cfgTableScope;
  var $cfgChartClubs;
  var $penalties; // Ligastrafen

  function __construct() {
    $this->_teamData = array();
  }

	/**
	 * Die Berechnung der Daten für die Tabellenfahrt. Da der Rückgabewert hier eine andere
	 * Struktur hat, läuft die Berechnung etwas anders.
	 * Was brauchen wir für eine Tabellenfahrt?
	 * - der Tabellenstand muss nach jeden Spieltag berechnet werden
	 * - die Position für für jedes Team in einem Array abgelegt
	 * - Die Punkte und Tore sind für die Ausgabe uninteressant
	 *
	 * @param tx_cfcleaguefe_util_league_TableProvider $tableProvider
	 */
	function generateChartData(&$tableProvider) {
		$this->setTableProvider($tableProvider);
		$this->_initTeams($tableProvider);
		$this->handlePenalties(); // Strafen können direkt berechnet werden
    $xyData = Array();
		$rounds = $tableProvider->getRounds();
    foreach($rounds As $round => $roundMatches) {
			$this->handleMatches($roundMatches);
			// Jetzt die Tabelle sortieren, dafür benötigen wir eine Kopie des Arrays
			$teamData = $this->_teamData;
			usort($teamData, array($this, 'compareTeams'));
			// Nun setzen wir die Tabellenstände
			reset($teamData);
			foreach($teamData As $position => $team) {
				if(in_array($team['clubId'], $this->getTableProvider()->getChartClubs()))
					$xyData[$team['teamName']][$round] = $position +1;
			}
		}
		// Issue 1880245: Chart auf der X-Achse bis Saisonende erweitern
		// Den höchsten absolvierten Spieltag ermitteln
		$lastRound = intval(array_pop(array_keys($rounds))) + 1;
		$maxRound = $this->getTableProvider()->getMaxRounds();
		$teamName = array_pop(array_keys($xyData));
		for( ; $lastRound <= $maxRound; $lastRound++) {
			// Es muss nur für ein Team ein weiterer Wert hinzugefügt werden
			$xyData[$teamName][$lastRound] = null;
		}
		return $xyData;
	}

	/**
	 * Für die Berechnung der Liga benötigen wir eine Datenlieferanten
	 * @param tx_cfcleaguefe_util_league_TableProvider $tableProvider
	 */
	function generateTable(&$tableProvider) {
		$this->setTableProvider($tableProvider);
		$this->_initTeams($tableProvider);
		$this->handlePenalties(); // Strafen können direkt berechnet werden

		$teamData = array();
		$rounds = $tableProvider->getRounds();
		foreach($rounds As $round => $roundMatches) {
			$this->handleMatches($roundMatches);
			// Jetzt die Tabelle sortieren, dafür benötigen wir eine Kopie des Arrays
			$teamData = $this->_teamData;
			usort($teamData, array($this, 'compareTeams'));
			// Nun setzen wir die Tabellenstände
			reset($teamData);
			for($i=0; $i < count($teamData); $i++) {
				$newPosition = $i +1;
				$team = $teamData[$i];
				if($this->_teamData[$team['teamId']]['position']) {
					$oldPosition = $this->_teamData[$team['teamId']]['position'];
					$this->_teamData[$team['teamId']]['oldposition'] = $oldPosition;
					$this->_teamData[$team['teamId']]['positionchange'] = $this->getPositionChange($oldPosition, $newPosition);
				}
				$this->_teamData[$team['teamId']]['position'] = $newPosition;
			}
		}
		usort($this->_teamData, array($this, 'compareTeams'));
		return $this->_teamData;
	}

	/**
	 * Die Ligastrafen werden in den Tabellenstand eingerechnet. Dies wird allerdings nur
	 * für die normale Tabelle gemacht. Sondertabellen werden ohne Strafen berechnet.
	 */
	function handlePenalties() {
		$penalties = $this->getTableProvider()->getPenalties();

    foreach($penalties As $penalty) {
      // Welches Team ist betroffen?
      if(array_key_exists($penalty->record['team'], $this->_teamData)) {
//    t3lib_div::debug($penalty, 'tx_cfcleaguefe_util_LeagueTable'); // TODO: Remove me!
        // Die Strafe wird für den View mit abgespeichert
        // Falls es eine Korrektur ist, dann nicht speichern
				if(!$penalty->isCorrection())
	        $this->_teamData[$penalty->record['team']]['penalties'][] = $penalty;
        // Die Punkte abziehen
        $this->_teamData[$penalty->record['team']]['points'] -= $penalty->record['points_pos'];
        $this->_teamData[$penalty->record['team']]['points2'] += $penalty->record['points_neg'];

        $this->addGoals($penalty->record['team'], ($penalty->record['goals_pos'] * -1), $penalty->record['goals_neg']);

        $this->_teamData[$penalty->record['team']]['matchCount'] += $penalty->record['matches'];
        $this->_teamData[$penalty->record['team']]['winCount'] += $penalty->record['wins'];
        $this->_teamData[$penalty->record['team']]['drawCount'] += $penalty->record['draws'];
        $this->_teamData[$penalty->record['team']]['loseCount'] += $penalty->record['loses'];

        // Den Zwangsabstieg tragen wir nur ein, damit der in die Sortierung eingeht 
        if($penalty->record['static_position'])
         $this->_teamData[$penalty->record['team']]['last_place'] = $penalty->record['static_position'];
      }
    }
  }

  /**
   * Die Spiele werden zum aktuellen Tabellenstand hinzugerechnet
   */
  function handleMatches(&$matches) {
    // Wir laufen jetzt über alle Spiele und legen einen Punktespeicher für jedes Team an
    foreach($matches As $match) {
    	if($match->isDummy()) continue; // Ignore Dummy-Matches
      // Wie ist das Spiel ausgegangen?
      $toto = $match->getToto();
			tx_rnbase_util_Misc::callHook('cfc_league_fe','leagueTable_handleMatches', 
				array('match' => &$match, 'teamdata'=>&$this->_teamData), $this);
      
      // Die eigentliche Punktezählung richtet sich nach dem Typ der Tabelle
      // Daher rufen wir jetzt die passende Methode auf
      switch($this->getTableProvider()->getTableType()) {
        case 1 :
          $this->_countHome($match, $toto);
          break;
        case 2 :
          $this->_countGuest($match, $toto);
          break;
        default:
          $this->_countStandard($match, $toto);
      }
    }
		unset($this->_teamData[0]); // Remove dummy data from teams without id
  }

	/**
	 * Zählt die Punkte für eine normale Tabelle
	 * @param tx_cfcleaguefe_models_match $match
	 * @param int $toto
	 */
	function _countStandard(&$match, $toto) {
		// Anzahl Spiele aktualisieren
		$homeId = $this->getTableProvider()->getTeamId($match->getHome());
		$guestId = $this->getTableProvider()->getTeamId($match->getGuest());
		$this->addMatchCount($homeId);
		$this->addMatchCount($guestId);

		if($toto == 0) { // Unentschieden
			$this->addPoints($homeId, $this->getTableProvider()->getPointsDraw());
			$this->addPoints($guestId, $this->getTableProvider()->getPointsDraw());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($homeId, $this->getTableProvider()->getPointsDraw());
				$this->addPoints2($guestId, $this->getTableProvider()->getPointsDraw());
			}

			$this->addDrawCount($homeId);
			$this->addDrawCount($guestId);
		}
		elseif($toto == 1) {  // Heimsieg
			$this->addPoints($homeId, $this->getTableProvider()->getPointsWin());
			$this->addPoints($guestId, $this->getTableProvider()->getPointsLoose());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($guestId, $this->getTableProvider()->getPointsWin());
			}

			$this->addWinCount($homeId);
			$this->addLoseCount($guestId);
		}
		else { // Auswärtssieg
			$this->addPoints($homeId, $this->getTableProvider()->getPointsLoose());
			$this->addPoints($guestId, $this->getTableProvider()->getPointsWin());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($homeId, $this->getTableProvider()->getPointsWin());
			}
			$this->addLoseCount($homeId);
			$this->addWinCount($guestId);
		}

		// Jetzt die Tore summieren
		$this->addGoals($homeId, $match->getGoalsHome(), $match->getGoalsGuest());
		$this->addGoals($guestId, $match->getGoalsGuest(), $match->getGoalsHome());
	}

  /**
   * Zählt die Punkte für eine Heimspieltabelle. Die Ergebnisse werden als nur für die 
   * Heimmannschaft gewertet.
	 * @param tx_cfcleaguefe_models_match $match
	 * @param int $toto
	 */
	function _countHome(&$match, $toto) {
		$homeId = $this->getTableProvider()->getTeamId($match->getHome());
		// Anzahl Spiele aktualisieren
		$this->addMatchCount($homeId);

		if($toto == 0) { // Unentschieden
			$this->addPoints($homeId, $this->getTableProvider()->getPointsDraw());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($homeId, $this->getTableProvider()->getPointsDraw());
			}
			$this->addDrawCount($homeId);
		}
		elseif($toto == 1) {  // Heimsieg
			$this->addPoints($homeId, $this->getTableProvider()->getPointsWin());
			$this->addWinCount($homeId);
		}
		else { // Auswärtssieg
			$this->addPoints($homeId, $this->getTableProvider()->getPointsLoose());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($homeId, $this->getTableProvider()->getPointsWin());
			}
			$this->addLoseCount($homeId);
		}
		// Jetzt die Tore summieren
		$this->addGoals($homeId, $match->getGoalsHome(), $match->getGoalsGuest());
	}

	/**
	 * Zählt die Punkte für eine normale Tabelle
	 * @param tx_cfcleaguefe_models_match $match
	 * @param int $toto
	 */
	function _countGuest(&$match, $toto) {
		$guestId = $this->getTableProvider()->getTeamId($match->getGuest());
		// Anzahl Spiele aktualisieren
		$this->addMatchCount($guestId);

		if($toto == 0) { // Unentschieden
			$this->addPoints($guestId, $this->getTableProvider()->getPointsDraw());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($guestId, $this->getTableProvider()->getPointsDraw());
			}
			$this->addDrawCount($guestId);
		}
		elseif($toto == 1) {  // Heimsieg
			$this->addPoints($guestId, $this->getTableProvider()->getPointsLoose());
			if($this->getTableProvider()->isCountLoosePoints()) {
				$this->addPoints2($guestId, $this->getTableProvider()->getPointsWin());
			}
			$this->addLoseCount($guestId);
		}
		else { // Auswärtssieg
			$this->addPoints($guestId, $this->getTableProvider()->getPointsWin());
			$this->addWinCount($guestId);
		}

		// Jetzt die Tore summieren
		$this->addGoals($guestId, $match->getGoalsGuest(), $match->getGoalsHome());
	}


  /**
   * Addiert Punkte zu einem Team
   */
  function addPoints($teamId, $points) {
    $this->_teamData[$teamId]['points'] = $this->_teamData[$teamId]['points'] + $points;
  }

  /**
   * Addiert negative Punkte zu einem Team. Diese Funktion wird nur im 2-Punkte-System
   * verwendet.
   */
  function addPoints2($teamId, $points) {
    $this->_teamData[$teamId]['points2'] = $this->_teamData[$teamId]['points2'] + $points;
  }

  /**
   * Addiert Tore zu einem Team
   */
  function addGoals($teamId, $goals1, $goals2) {
    $this->_teamData[$teamId]['goals1'] = $this->_teamData[$teamId]['goals1'] + $goals1;
    $this->_teamData[$teamId]['goals2'] = $this->_teamData[$teamId]['goals2'] + $goals2;
    $this->_teamData[$teamId]['goals_diff'] = $this->_teamData[$teamId]['goals1'] - $this->_teamData[$teamId]['goals2'];
  }

  /**
   * Addiert die absolvierten Spiele zu einem Team
   */
  function addMatchCount($teamId) {
    $this->_teamData[$teamId]['matchCount'] = $this->_teamData[$teamId]['matchCount'] + 1;
  }

  function addWinCount($teamId) {
    $this->_teamData[$teamId]['winCount'] = $this->_teamData[$teamId]['winCount'] + 1;
  }
  function addDrawCount($teamId) {
    $this->_teamData[$teamId]['drawCount'] = $this->_teamData[$teamId]['drawCount'] + 1;
  }
  function addLoseCount($teamId) {
    $this->_teamData[$teamId]['loseCount'] = $this->_teamData[$teamId]['loseCount'] + 1;
  }

  /**
   * Lädt die Namen der Teams in der Tabelle
   * @param tx_cfcleaguefe_models_competition $tableProvider
   */
  function _initTeams(tx_cfcleaguefe_util_league_TableProvider $tableProvider) {
    // Wir laden die Teams aus der Liga
//    $uids = $league->record['teams'];
//    $where = 'uid IN (' . $uids .')';
//    $teams = tx_rnbase_util_DB::queryDB('*','tx_cfcleague_teams',$where,
//              '','sorting','tx_cfcleaguefe_models_team',0);

		$teams = $tableProvider->getTeams();
		foreach($teams As $team) {
			$teamId = $tableProvider->getTeamId($team);
			if(!$teamId) continue; // Ignore teams without given id
//			if($team->isDummy()) continue; // Ignore dummy teams
			if(array_key_exists($teamId, $this->_teamData)) continue;
			$this->_teamData[$teamId]['team'] = $team;
			$this->_teamData[$teamId]['teamId'] = $teamId;
			$this->_teamData[$teamId]['teamName'] = $team->record['name'];
			$this->_teamData[$teamId]['teamNameShort'] = $team->record['short_name'];
			$this->_teamData[$teamId]['clubId'] = $team->record['club'];
			$this->_teamData[$teamId]['points'] = 0;
			// Bei 3-Punktssystem muss mit -1 initialisiert werden, damit der Marker später ersetzt wird
			$this->_teamData[$teamId]['points2'] = ($tableProvider->isCountLoosePoints()) ? 0 : -1;
			$this->_teamData[$teamId]['goals1'] = 0;
			$this->_teamData[$teamId]['goals2'] = 0;
			$this->_teamData[$teamId]['goals_diff'] = 0;
			$this->_teamData[$teamId]['position'] = 0;
			$this->_teamData[$teamId]['oldposition'] = 0;
			$this->_teamData[$teamId]['positionchange'] = 'EQ';

			$this->_teamData[$teamId]['matchCount'] = 0;
			$this->_teamData[$teamId]['winCount'] = 0;
			$this->_teamData[$teamId]['drawCount'] = 0;
			$this->_teamData[$teamId]['loseCount'] = 0;

			// Muss das Team hervorgehoben werden?
			$markClubs = $tableProvider->getMarkClubs();
			if(count($markClubs)) {
				$this->_teamData[$teamId]['markClub'] = in_array($team->record['club'], $markClubs) ? 1 : 0;
			}
		}
//      t3lib_div::debug($this->_teamData,'Vereine');
	}
	/**
	 * Returns position change, either UP or DOWN or EQ.
	 *
	 * @param int $oldPosition
	 * @param int $newPosition
	 * @return string UP, DOWN or EQ
	 */
	protected function getPositionChange($oldPosition, $newPosition) {
		return $oldPosition == $newPosition ? 'EQ' : ($oldPosition > $newPosition ? 'UP' : 'DOWN');
	}
	

	function _getDefaults($configurations, $league) {
		$defaults['pointsystem'] = $league->record['point_system'];
		$defaults['pointsystemInput'] = $configurations->get('pointSystemSelectionInput');
		$defaults['chartclubs'] = $configurations->get('chartClubs');
		$defaults['markclubs'] = $configurations->get('markClubs');
		$defaults['tabletype'] = $configurations->get('tabletype');
		$defaults['tabletypeInput'] = $configurations->get('tabletypeSelectionInput');
		$defaults['tablescope'] = $configurations->get('tablescope');
		$defaults['tablescopeInput'] = $configurations->get('tablescopeSelectionInput');
    $defaults['penalties'] = $league->getPenalties();
		return $defaults;
  }

  /**
   * Initialisiert die Instanz mit den notwendigen Daten
   * @param tx_lib_parameters $parameters
   * @param array $defaults
   */
	function _initConfig(&$parameters, &$defaults) {
		// Das Punktesystem kommt aus der Liga, kann aber über einen Parameter verändert werden
		$this->cfgPointSystem = $defaults['pointsystem'];
		// Das das Punktesystem geändert werden?
		if($defaults['pointSystemInput']) {
			// Prüfen, ob der Parameter im Request liegt
			if($parameters->offsetGet('pointsystem'))
				$this->cfgPointSystem = intval($parameters->offsetGet('pointsystem'));
		}

		$this->cfgChartClubs = t3lib_div::intExplode(',',$defaults['chartclubs']);
		$this->cfgMarkClubs = t3lib_div::intExplode(',',$defaults['markclubs']);
		$this->cfgTableType = $defaults['tabletype'];
//t3lib_div::debug($configurations);
		// Wird der TableType per SelectBox angeboten?
		if($defaults['tabletypeInput']) {
			$this->cfgTableType = $parameters->offsetGet('tabletype') ? $parameters->offsetGet('tabletype') : $this->cfgTableType;
		}

		// Der TableScope wirkt sich auf die betrachteten Spiele aus
		$this->cfgTableScope = $defaults['tablescope'];
		if($defaults['tablescopeInput']) {
			$this->cfgTableScope = $parameters->offsetGet('tablescope') ? $parameters->offsetGet('tablescope') : $this->cfgTableScope;
		}

		if($this->cfgPointSystem == '0') { // 3-Punkt
			$this->cfgPointsDraw = 1;
			$this->cfgPointsWin = 3;
			$this->cfgPointsLose = 0;
		}
		elseif($this->cfgPointSystem == '1') { // 2-Punkt
			$this->cfgPointsDraw = 1;
			$this->cfgPointsWin = 2;
			$this->cfgPointsLose = 0;
		}
		else {
			t3lib_div::debug($this->cfgPointSystem,'Error: Unknown Pointsystem-ID');
		}
		// Die Ligastrafen laden
		$this->penalties = $defaults['penalties'];
	}
  function setTableProvider(&$tableProvider) {
  	$this->tableProvider = $tableProvider;
  }
  /**
   * The data provider
   *
   * @return tx_cfcleaguefe_util_league_TableProvider
   */
  function getTableProvider() {
  	return $this->tableProvider;
  }

	/**
	 * Funktion zur Sortierung der Tabellenzeilen
	 */
	function compareTeams($t1, $t2) {
		// Zwangsabstieg prüfen
		if($t1['last_place']) return 1;
		if($t2['last_place']) return -1;
	
		if($t1['points'] == $t2['points']) {
			// Im 2-Punkte-Modus sind die Minuspunkte auschlaggebend
			// da sie im 3-PM immer identisch sein sollten, können wir immer testen
			if($t1['points2'] == $t2['points2']) {
				// Jetzt die Tordifferenz prüfen
				$t1diff = $t1['goals1'] - $t1['goals2'];
				$t2diff = $t2['goals1'] - $t2['goals2'];
				if($t1diff == $t2diff) {
					// Jetzt zählen die mehr geschossenen Tore
					if($t1['goals1'] == $t2['goals1'])
						return 0; // Punkt und Torgleich
					return $t1['goals1'] > $t2['goals1'] ? -1 : 1;
				}
				return $t1diff > $t2diff ? -1 : 1;
			}
			// Bei den Minuspunkten ist weniger mehr
			return $t1['points2'] < $t2['points2'] ? -1 : 1;
		}
		return $t1['points'] > $t2['points'] ? -1 : 1;
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cfc_league_fe/util/class.tx_cfcleaguefe_util_LeagueTable.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cfc_league_fe/util/class.tx_cfcleaguefe_util_LeagueTable.php']);
}

?>